/*----------------------------------------------------------------------------*
 | Headerfile Callback1Detail.h                                               |
 |                                                                            |
 | Version 1.20                                                               |
 |                                                                            |
 |                                                                            |
 | BriDGe::Callback: a tiny library that provides callback functionality.     |
 | Copyright (C) 2002  Bram de Greve                                          |
 |                                                                            |
 | This library is free software; you can redistribute it and/or              |
 | modify it under the terms of the GNU Lesser General Public                 |
 | License as published by the Free Software Foundation; either               |
 | version 2.1 of the License, or (at your option) any later version.         |
 |                                                                            |
 | This library is distributed in the hope that it will be useful,            |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of             |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          |
 | Lesser General Public License for more details.                            |
 |                                                                            |
 | You should have received a copy of the GNU Lesser General Public           |
 | License along with this library; if not, write to the Free Software        |
 | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  |
 |                                                                            |
 |                                                                            |
 | Contact by e-mail: Bram de Greve <bdegreve@mail.be>                        |
 |                                                                            |
 |                                                                            |
 | History:                                                                   |
 | June 7th, 2002: Splitting Callback1 to multiple files.                     |
 |                 Creation of Callback1Detail.h                              |
 | June 8th, 2002: Revamping the code.                                        |
 |                 Putting stuff under GNU LGPL.                              |
 | June 9th, 2002: Getting rid of boost::shared_ptr so we look less silly,    |
 |                 Getting rid of makeCallback since we use templated         |
 |                 constructors now.                                          |
 |                 Making the call of an empty callback a little more safe    |
 |                 by throwing an exception CallbackEmptyCall.                |
 |                 Getting rid of operator bool and renaming isEmpty to empty.|
 |                 And some other stuff too.  Thank you Dirk Gerrits!         |
 |                 Soit, we have now officially reached version 2.00!         |
 |                                                                            |
 | I want to thank Altair, Jaap Suter & especially Dirk Gerrits (Who has      |
 | spent an entire afternoon on this to help me.  It was a waste of time for  |
 | both of us, but we had a lot of fun, didn't we? :)                         |
 *----------------------------------------------------------------------------*/

#ifndef BDG_GUARDIAN_OF_INCLUSION_CALLBACK1DETAIL_H
#define BDG_GUARDIAN_OF_INCLUSION_CALLBACK1DETAIL_H



// --- NEW INTERFACES ----------------------------------------------------------

namespace BDG
{
    
namespace Callback1_Detail
{

// Base class for dispatchers:
   
template<typename P1>
class DispBase
{
public:

    virtual void operator()(P1 a_p1) const = 0;
    virtual DispBase* clone() const = 0;
};



// Dispatcher to a non-member function:

template<typename P1, typename FP1>
class DispFunction: public DispBase<P1>
{
private:

    typedef void (*Function) (FP1);
    Function m_function;

public:

    DispFunction(Function a_function):
        m_function(a_function)
    {
    }

    void operator()(P1 a_p1) const
    {
        (*m_function)(a_p1); 
    }

    DispBase<P1>* clone() const
    {
        return new DispFunction<P1, FP1>(*this);
    }
};



// Dispatcher to Object/Method pair:

template <class Object, typename P1, typename MP1>
class DispMethod: public DispBase<P1>
{
private:

    typedef void (Object::*Method) (MP1);
    Object* m_object;
    Method m_method;

public:

    DispMethod(Object* a_object, Method a_method): 
        m_object(a_object), 
        m_method(a_method)
    {
    }

    void operator()(P1 a_p1) const
    {
        (m_object->*m_method)(a_p1); 
    }
    
    DispBase<P1>* clone() const
    {
        return new DispMethod<Object, P1, MP1>(*this);
    }
};



// Dispatcher to Object/(const Method) pair:

template <class Object, typename P1, typename MP1>
class DispMethodConst: public DispBase<P1>
{
private:

    typedef void (Object::*Method) (MP1) const;
    Object* m_object;
    Method m_method;

public:

    DispMethodConst(Object* a_object, Method a_method):
        m_object(a_object), 
        m_method(a_method)
    {
    }

    void operator()(P1 a_p1) const 
    {
        (m_object->*m_method)(a_p1); 
    }

    DispBase<P1>* clone() const
    {
        return new DispMethodConst<Object, P1, MP1>(*this);
    }
};



} // namespace Callback1_Detail

} // namespace BDG

#endif // Guardian of Inclusion


// --- END OF FILE Callback1Detail.h -------------------------------------------
